import requests
import requests.exceptions


class OnfidoError(Exception):
    pass

class OnfidoUnknownError(OnfidoError):
    pass

class OnfidoInvalidSignatureError(OnfidoError):
    pass

class OnfidoRequestError(OnfidoError):
    pass

class OnfidoServerError(OnfidoError):
    pass

class OnfidoConnectionError(OnfidoError):
    pass

class OnfidoTimeoutError(OnfidoError):
    pass

def error_decorator(func):
    def wrapper(*args, **kwargs):
        try:
           return func(*args, **kwargs)

        except requests.HTTPError as e:
            if e.response.status_code >= 500:
                raise OnfidoServerError() from e
            else:
                raise OnfidoRequestError() from e

        except requests.Timeout as e:
            raise OnfidoTimeoutError(e)

        except requests.ConnectionError as e:
            raise OnfidoConnectionError(e)

        except requests.RequestException as e:
            raise OnfidoUnknownError(e)
    return wrapper




from ..resource import Resource


class LivePhotos(Resource):
    def upload(self, sample_file:bytes, request_body):

        return self.upload_request("live_photos/", sample_file, **request_body)

    def find(self, live_photo_id:str):
        return self.get(f"live_photos/{live_photo_id}")

    def all(self, applicant_id:str):
        payload = {"applicant_id": applicant_id}
        return self.get("live_photos/", payload=payload)

    def download(self, live_photo_id:str):
        return self.download_request(f"live_photos/{live_photo_id}/download")



from enum import Enum

class Region(Enum):
    EU = "https://api.onfido.com/v3/"
    US = "https://api.us.onfido.com/v3/"
    CA = "https://api.ca.onfido.com/v3/"

    @property
    def region_url(self):
        return self.value



from ..resource import Resource


class Webhooks(Resource):
    def create(self, request_body:dict):
        return self.post("webhooks", **request_body)

    def all(self):
        return self.get("webhooks")

    def find(self, webhook_id:str):
        return self.get(f"webhooks/{webhook_id}")

    def edit(self, webhook_id:str, request_body:dict):
        return self.put(f"webhooks/{webhook_id}", request_body)

    def delete(self, webhook_id:str):
        self.delete_request(f"webhooks/{webhook_id}")



from ..resource import Resource


class Checks(Resource):
    def create(self, request_body:dict):
        return self.post("checks/", **request_body)

    def find(self, check_id:str):
        return self.get(f"checks/{check_id}")

    def all(self, applicant_id:str):
        payload = {"applicant_id": applicant_id}
        return self.get("checks/", payload=payload)

    def resume(self, check_id:str):
        self.post(f"checks/{check_id}/resume")



from ..resource import Resource


class Applicants(Resource):
    def create(self, request_body:dict):
        return self.post("applicants/", **request_body)

    def update(self, applicant_id:str, request_body:dict):
        return self.put(f"applicants/{applicant_id}", request_body)

    def find(self, applicant_id:str):
        return self.get(f"applicants/{applicant_id}")

    def delete(self, applicant_id:str):
        self.delete_request(f"applicants/{applicant_id}")

    def all(self, **user_payload:dict):
        payload = {"include_deleted": False, "per_page": 20, "page": 1}
        payload.update(user_payload)
        return self.get("applicants", payload=payload)

    def restore(self, applicant_id:str):
        self.post(f"applicants/{applicant_id}/restore")



from .resources.applicants import Applicants
from .resources.documents import Documents
from .resources.address_picker import Addresses
from .resources.checks import Checks
from .resources.reports import Reports
from .resources.live_photos import LivePhotos
from .resources.live_videos import LiveVideos
from .resources.webhooks import Webhooks
from .resources.sdk_tokens import SdkToken
from .regions import Region

class Api:
    def __init__(self, api_token, base_url=Region.EU):
        self.applicant = Applicants(api_token, base_url)
        self.document = Documents(api_token, base_url)
        self.address = Addresses(api_token, base_url)
        self.check = Checks(api_token, base_url)
        self.report = Reports(api_token, base_url)
        self.sdk_token = SdkToken(api_token, base_url)
        self.webhook = Webhooks(api_token, base_url)
        self.live_photo = LivePhotos(api_token, base_url)
        self.live_video = LiveVideos(api_token, base_url)



import requests
from .onfido_download import OnfidoDownload
from .exceptions import error_decorator, OnfidoUnknownError
from .mimetype import mimetype_from_name

class Resource:
    def __init__(self, api_token, base_url):
        self.api_token = api_token
        self.base_url = base_url

    @property
    def url(self):
        return getattr(self.base_url, "region_url", self.base_url)

    def build_url(self, path):
        return self.url + path

    @property
    def _headers(self):
        return {
            "Authorization": f"Token token={self.api_token}"
        }

    def handle_response(self, response):
        response.raise_for_status()
        if response.status_code == 204:
            return None

        try:
            return response.json()
        except ValueError as e:
            raise OnfidoUnknownError("Onfido returned invalid JSON") from e

    @error_decorator
    def upload_request(self, path, file, **request_body):
        files = {
            'file': (file.name, file, mimetype_from_name(file.name))
        }
        
        response = requests.post(self.build_url(path), data=request_body, files = files, headers=self._headers)

        return self.handle_response(response)

    @error_decorator
    def post(self, path, **request_body):
        response = requests.post(self.build_url(path), json=request_body,
                                 headers=self._headers)

        return self.handle_response(response)

    @error_decorator
    def put(self, path, data=None):
        response = requests.put(self.build_url(path), json=data,
                                headers=self._headers)

        return self.handle_response(response)

    @error_decorator
    def get(self, path, payload=None):
        response = requests.get(self.build_url(path), headers=self._headers, params=payload)

        return self.handle_response(response)

    @error_decorator
    def download_request(self, path):
        response = requests.get(self.build_url(path), headers=self._headers)

        response.raise_for_status()

        try:
            return OnfidoDownload(response)
        except ValueError as e:
            raise OnfidoUnknownError("Onfido returned invalid JSON") from e


    @error_decorator
    def delete_request(self, path):
        response = requests.delete(self.build_url(path), headers=self._headers)

        return self.handle_response(response)




import hmac
import hashlib
import json
from .exceptions import OnfidoInvalidSignatureError

class WebhookEventVerifier:
    def __init__(self, webhook_token):
        self.webhook_token = webhook_token

    def read_payload(self, raw_event, signature):
        # Compute the the actual HMAC signature from the raw request body.
        event_signature = hmac.new(key=self.webhook_token.encode("utf-8"),
                                   msg=raw_event.encode("utf-8"),
                                   digestmod=hashlib.sha256).hexdigest()

        # Compare the signatures (prevent against timing attacks).
        if not hmac.compare_digest(signature, event_signature):
            raise OnfidoInvalidSignatureError()

        return json.loads(raw_event)["payload"]



from ..resource import Resource


class Reports(Resource):
    def find(self, report_id:str):
        return self.get(f"reports/{report_id}")

    def all(self, check_id:str):
        payload = {"check_id": check_id}
        return self.get("reports/", payload=payload)

    def resume(self, report_id:str):
        return self.post(f"reports/{report_id}/resume")

    def cancel(self, report_id:str):
        self.post(f"reports/{report_id}/cancel")



from ..resource import Resource


class LiveVideos(Resource):
    def find(self, live_video_id:str):
        return self.get(f"live_videos/{live_video_id}")

    def all(self, applicant_id:str):
        payload = {"applicant_id": applicant_id}
        return self.get("live_videos/", payload=payload)

    def download(self, live_video_id:str):
        return self.download_request(f"live_videos/{live_video_id}/download")

    def download_frame(self, live_video_id:str):
        return self.download_request(f"live_videos/{live_video_id}/frame")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['onfido', 'onfido.resources']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.24.0,<3.0.0']

setup_kwargs = {
    'name': 'onfido-python',
    'version': '0.4.0',
    'description': 'The official wrapper for the Onfido API',
    'long_description': '# Onfido Python Client Library\n\n:warning: **IN BETA**\n\n[onfido-python on PyPI](https://pypi.org/project/onfido-python/)\n\nVersion 0.4.0\n\nThe official wrapper for Onfido\'s API. Refer to the full [API documentation](https://documentation.onfido.com) for details of expected requests and responses.\n\nThis project supersedes the automatically generated [api-python-client](https://github.com/onfido/api-python-client) library (`onfido` in PyPI).\n\n## Installation\n\n`pip install onfido-python`\n\n:warning: Having the old `onfido` package installed at the same time will cause errors.\n\n## Usage\n\nMake API calls by using an instance of the `Api` class and providing your API\ntoken:\n\n```python\nimport onfido\n\napi = onfido.Api("<YOUR_API_TOKEN>")\n```\n\n### Regions\n\nSet the region in the API instance using the `base_url` parameter.\n\nThe library will use the default base URL (api.onfido.com) for the EU region, if\nno region is specified.\n\nTo specify the US region do:\n\n```python\nfrom onfido.regions import Region\n\napi = onfido.Api("<YOUR_API_TOKEN>", base_url=Region.US)\n```\n\nTo specify the CA region do:\n\n```python\nfrom onfido.regions import Region\n\napi = onfido.Api("<YOUR_API_TOKEN>", base_url=Region.CA)\n```\n\nSee https://documentation.onfido.com/#regions for supported regions.\n\n### Response format\n\nThe Python library will return data directly from the API.\n\nSee https://documentation.onfido.com/#request,-response-format for more\ninformation.\n\n### Resources\n\nAll resources share the same interface when making API calls. For example, use\n`.create` to create a resource, `.find` to find one, and `.all` to fetch all\nresources.\n\n#### Applicants\n\nApplicants are the object upon which Onfido checks are performed.\n\n```python\napi.applicant.create(params)  # => Creates an applicant\napi.applicant.update("<APPLICANT_ID>", params)  # => Updates an applicant\napi.applicant.delete("<APPLICANT_ID>")  # => Schedule an applicant for deletion\napi.applicant.restore("<APPLICANT_ID>") # => Restore an applicant scheduled for deletion\napi.applicant.find("<APPLICANT_ID>")  # => Finds a single applicant\napi.applicant.all()  # => Returns all applicants\n```\n\n`applicant.all()` takes the following optional arguments:\n\n`include_deleted=true`: include applicants scheduled for deletion.\n`per_page`: set the number of results per page. Defaults to 20.\n`page`: return specific pages. Defaults to 1.\n\n**Note:** Calling `api.applicant.delete` adds the applicant and all associated\ndocuments, photos, videos, checks, reports and analytics data to our deletion\nqueue. Please read https://documentation.onfido.com/#delete-applicant for more\ninformation.\n\n#### Documents\n\nSome report types require identity documents (passport, driving licence etc.) in order to be processed.\n\n```python\nrequest_body = {"applicant_id": "<APPLICANT_ID>", "document_type": "passport"}\nsample_file = open("<FILE_NAME>", "rb")\n\napi.document.upload(sample_file, request_body)   # => Uploads a document\napi.document.find("<DOCUMENT_ID>")      # => Finds a document\napi.document.download("<DOCUMENT_ID>")  # => Downloads a document as a binary data\napi.document.all("<APPLICANT_ID>")      # => Returns all documents belonging to an applicant\n```\n\nSee https://documentation.onfido.com/#document-types for example document types.\n\n#### Live Photos\n\nLive photos are images of the applicant’s face, typically taken at the same time as documents are provided. These photos are used to perform Facial Similarity Photo reports on the applicant.\n\n```python\nrequest_body = {"applicant_id": "<APPLICANT_ID>", "advanced_validation": "True"}\nsample_file = open("<FILE_NAME>", "rb")\n\napi.live_photo.upload(sample_file, request_body)   # => Uploads a live photo\napi.live_photo.find("<LIVE_PHOTO_ID>")      # => Finds a live photo\napi.live_photo.download("<LIVE_PHOTO_ID>")  # => Downloads a live photo as binary data\napi.live_photo.all("<APPLICANT_ID>")        # => Returns all live photos belonging to an applicant\n```\n\n#### Checks\n\nChecks are performed on an applicant. Depending on the type of check you wish to perform, different information will be required when you create an applicant. A check consists of one or more reports.\n\n```python\nrequest_body = {"applicant_id": "12345", "report_names": ["document", "facial_similarity_photo"]}\n\napi.check.create(request_body) # => Creates a check\napi.check.find("<CHECK_ID>")    # => Finds a check\napi.check.resume("<CHECK_ID>")  # => Resumes a paused check\napi.check.all("<APPLICANT_ID>") # => Returns all an applicant\'s checks\n```\n\n#### Reports\n\nReports provide details of the results of some part of a check. They are\ncreated when a check is created, so the Onfido API only provides support for\nfinding and listing them. For paused reports specifically, additional support for resuming and\n cancelling reports is also available.\n\n```python\napi.report.find("<REPORT_ID>")    # => Finds a report\napi.report.resume("<REPORT_ID>")  # => Resumes a paused report\napi.report.all("<CHECK_ID>")      # => Returns all the reports in a check\napi.report.cancel("<REPORT_ID>")  # => Cancels a paused report\n```\n\n#### Address Lookups\n\nOnfido provides an address lookup service, to help ensure well-formatted\naddresses are provided when creating applicants. To search for addresses\nby postcode, use:\n\n```python\napi.address.pick("SW46EH") # => Returns all addresses in a given postcode\n```\n\n#### Webhook Endpoints\n\nOnfido allows you to set up and view your webhook endpoints via the API, as well\nas through the Dashboard.\n\n```python\nrequest_body = {\n  "url": "https://<URL>",\n  "events": [\n    "report.completed",\n    "check.completed"\n  ]\n}\n\napi.webhook.create(request_body) # => Registers a webhook\napi.webhook.find("<WEBHOOK_ID>")  # => Finds a single webhook\napi.webhook.edit("<WEBHOOK_ID>", new_webhook_details) # => Edits a webhook\napi.webhook.delete("<WEBHOOK_ID>") # => Deletes a webhook\napi.webhook.all() # => Returns all webhooks\n```\n\n##### Webhook Verification\n\nA webhook event is valid if the signature from the header is equal to the\nexpected signature you compute for it.\n\n```python\nevent = verifier.read_payload(raw_event, signature)\n```\n\nSee https://documentation.onfido.com/#verifying-webhook-signatures for more information.\n\n#### SDK Tokens\n\nOnfido allows you to generate JSON Web Tokens via the API in order to authenticate\nwith Onfido\'s SDKs.\n\n```python\nrequest_body = {"applicant_id": "<APPLICANT_ID>", "application_id": "<APPLICATION_ID>"}\n\napi.sdk_token.create(request_body) # => Creates an SDK token\n```\n\n### Error Handling\n\n- `OnfidoServerError` is raised whenever Onfido returns a `5xx` response\n- `OnfidoRequestError` is raised whenever Onfido returns a `4xx` response\n- `OnfidoInvalidSignatureError` is raised whenever a signature from the header is not equal to the expected signature you compute for it\n- `OnfidoTimeoutError` is raised if a timeout occurs\n- `OnfidoConnectionError` is raised whenever any other network error occurs\n- `OnfidoUnknownError` is raised whenever something unexpected happens\n\n## Contributing\n\n1. Fork it ( https://github.com/onfido/onfido-python/fork )\n2. Create your feature branch (`git checkout -b my-new-feature`)\n3. Commit your changes (`git commit -am \'Add some feature\'`)\n4. Push to the branch (`git push origin my-new-feature`)\n5. Create a new Pull Request\n',
    'author': 'Ben Ahmady',
    'author_email': 'ben.ahmady@onfido.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/onfido/onfido-python',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4',
}


setup(**setup_kwargs)



from ..resource import Resource


class Documents(Resource):
    def upload(self, sample_file, request_body):
        return self.upload_request("documents/", sample_file, **request_body)

    def find(self, document_id:str):
        return self.get(f"documents/{document_id}")

    def all(self, applicant_id:str):
        return self.get(f"documents?applicant_id={applicant_id}")

    def download(self, document_id:str):
        return self.download_request(f"documents/{document_id}/download")
